<?php 

	require_once(dirname(__FILE__) . '/ETumblrJson.php');
	require_once(dirname(__FILE__) . '/ETumblrParams.php');
	
	/**
	 * Implements a base data provider based on tumblr posts
	 * 
	 * @author Dimitar Dinchev, Instinctiv Inc (http://instinctiv.com/)
	 *
	 */
	class ETumblrDataProvider extends CDataProvider
	{
		protected $_tumblrLib;
		protected $_tumblrParams;
		
		public function __construct(ETumblrJson $tumblrLib) {
			$this->_tumblrLib = $tumblrLib;
		}
		
		public function setTumblrParams(ETumblrParams $params) {
			$this->_tumblrParams = $params;
			return $this;
		}
		
		/**
		 * 
		 * @return ETumblrParams
		 */
		public function getTumblrParams() {
			if (!isset($this->_tumblrParams))
				$this->_tumblrParams = new ETumblrParams;
			
			return $this->_tumblrParams;
		}
		
		/**
		 * Fetches the data from the persistent data storage.
		 * @return array list of data items
		 */
		protected function fetchData()
		{
			if (($pagination = $this->getPagination()) !== false) {
				$pagination->setItemCount($this->getTotalItemCount());
				$this->getTumblrParams()->setStart($pagination->getOffset());
				$this->getTumblrParams()->setNum($pagination->getLimit());
			}
			
			return $this->_tumblrLib->setParams($this->getTumblrParams())->getPosts();
		}

		/**
		 * Fetches the data item keys from the persistent data storage.
		 * @return array list of data item keys.
		 */
		protected function fetchKeys()
		{
			$keys = array();
			foreach ($this->getData() as $i => $data) {
				$keys[] = $data['id'];
			}
			
			return $keys;
		}
		
		/**
		 * Calculates the total number of data items.
		 * @return integer the total number of data items.
		 */
		protected function calculateTotalItemCount() {
			return $this->_tumblrLib->setParams($this->getTumblrParams())->getTotalPostsCount();
		}
	}