<?php
	/**
	 * 
	 * Tumblr PHP library to pull posts. It provides easy interface to integrate pagination. 
	 * 
	 * @author Dimitar Dinchev, Instinctiv Inc (http://instinctiv.com/) 
	 *
	 */
	class ETumblrJson
	{
		const TUMBLR_JS_VAR_NAME_LENGTH = 10;
		
		protected $_connectionTimeout = 5;
		protected $_requestTimeout = 30;
		
		protected $_tumblrId;
		
		protected $_validTumblrTypes = array('regular', 'quote', 'photo', 'link', 'conversation', 'video', 'audio');
		protected $_validTumblrFilters = array('text', 'none');
		
		protected $_params;
		protected $_response;
		
		public function __construct($tumblrId) {
			if (!function_exists('curl_init'))
				throw new Exception('The ETumblrJson needs the CURL PHP extension.');
			
			$this->_tumblrId = $tumblrId;
		}
		
		public function setParams(ETumblrParams $params) 
		{
			if ($this->_params == $params)
				return $this;
			
			// params have changed
			$this->_response = null;
			$this->_params = $params;
			
			return $this;
		}
		
		/**
		 * 
		 * @return ETumblrParams
		 */
		public function getParams() {
			return $this->_params;
		}
		
		public function getPosts()
		{
			if ($this->_response === null)
				$this->read();
			
			return $this->_response['posts'];
		}
		
		public function getTotalPostsCount()
		{
			if ($this->_response === null)
				$this->read();
			
			return (int)$this->_response['posts-total'];
		}
		
		protected function read() 
		{
			$apiUrl = 'http://' . $this->_tumblrId . '.tumblr.com/api/read/json';
			
			$params = $this->_params->getAsArray();
			
			if (isset($params['type']) && !in_array($params['type'], $this->_validTumblrTypes))
				throw new ETumblrException(sprintf('Invalid tumblr type "%s", valid types: "%s"', $params['type'], implode($this->_validTumblrTypes)));
			
			if (isset($params['filter']) && !in_array($params['filter'], $this->_validTumblrFilters))
				throw new ETumblrException(sprintf('Invalid tumblr filter "%s", valid filters: "%s"', $params['filter'], implode($this->_validTumblrFilters)));
			
			$ch = $this->curlInit($apiUrl, $params);
			
			$json = preg_replace('#var tumblr_api_read = (.*);#', '$1', $this->curlExec($ch));
			
			$decodedResponse = json_decode($json, true);
			
			$this->_response = $decodedResponse;
			return $this;
		}
		
		protected function curlInit($url, $params)
		{
			$urlParams = http_build_query($params, null, '&');
			$url .= '?' . $urlParams;
			$ch = curl_init($url);
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
			curl_setopt($ch, CURLOPT_TIMEOUT, $this->_requestTimeout);
			curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, $this->_connectionTimeout);
			return $ch;
		}
		
		protected function curlExec($ch)
		{
			$resp = curl_exec($ch);
			
			if ($resp === false) {
				$error = curl_error($ch);
				$errNo = curl_errno($ch);
				curl_close($ch);
				// might be a good idea to implement own ETumblrJsonException
				throw new ETumblrException($error, $errNo);				
			}
			
			return $resp;
		}
	}
	
	/**
	 * Generic exception object so that the library remains independent of Yii
	 * @author d_dinchev, Instinctiv Inc (http://instinctiv.com/) 
	 */
	class ETumblrException { }